package edu.uoc.android.socialplan.activities.general;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.NonNull;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.util.Log;
import android.view.View;
import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.facebook.AccessToken;
import com.facebook.login.LoginManager;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.EmailAuthProvider;
import com.google.firebase.auth.FacebookAuthCredential;
import com.google.firebase.auth.FacebookAuthProvider;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.messaging.FirebaseMessaging;
import com.squareup.picasso.Picasso;

import org.greenrobot.greendao.database.Database;

import java.util.List;

import butterknife.BindView;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.registro.RegistroActivity;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.application.BaseActivity;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.constants.TipoLoginUserEnum;
import edu.uoc.android.socialplan.constants.TiposPlanes;
import edu.uoc.android.socialplan.fragments.HomeFragment;
import edu.uoc.android.socialplan.fragments.NotificacionesFragment;
import edu.uoc.android.socialplan.fragments.NuevoPlanFragment;
import edu.uoc.android.socialplan.fragments.PerfilFragment;
import edu.uoc.android.socialplan.fragments.RedFragment;
import edu.uoc.android.socialplan.models.Comentario;
import edu.uoc.android.socialplan.models.DaoMaster;
import edu.uoc.android.socialplan.models.DaoSession;
import edu.uoc.android.socialplan.models.Plan;
import edu.uoc.android.socialplan.models.Usuario;
import edu.uoc.android.socialplan.utils.GeneralUtils;
import edu.uoc.android.socialplan.utils.ImageUtils;
import edu.uoc.android.socialplan.utils.NetworkUtils;
import edu.uoc.android.socialplan.views.CustomProgressBarDialog;

public class MainActivity extends BaseActivity
        implements NavigationView.OnNavigationItemSelectedListener {

    @BindView(R.id.toolbar) public Toolbar toolbar;
    @BindView(R.id.drawer_layout) public DrawerLayout drawer;
    @BindView(R.id.nav_view) public NavigationView navigationView;

    public ImageView userImage;
    public TextView userName;
    public ActionBarDrawerToggle toggle;

    private static final String TAG = "MainActivity";

    private SharedPreferences prefs;
    private DaoSession daoSession;
    private DatabaseReference realtimeDB;
    private CustomProgressBarDialog progressBarDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setSupportActionBar(toolbar);

        toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.addDrawerListener(toggle);
        toggle.syncState();

        navigationView.setNavigationItemSelectedListener(this);

        //Cabecera
        final View navHeader = navigationView.getHeaderView(0);
        userImage = navHeader.findViewById(R.id.user_image);
        userName = navHeader.findViewById(R.id.user_name);

        prefs = ((App)getApplication()).getPrefs();
        daoSession = ((App)getApplication()).getDaoSession();
        realtimeDB = ((App)getApplication()).getRealtimeDB();

        progressBarDialog = GeneralUtils.getCustomProgressBarDialog(MainActivity.this);


        updateUI();

        final ViewGroup viewGroup = (ViewGroup) ((ViewGroup) this
                .findViewById(android.R.id.content)).getChildAt(0);

        final Handler handler = new Handler();
        handler.post(new Runnable() {
            public void run() {
                if (!NetworkUtils.isNetworkConnected(getApplicationContext())) {
                    Snackbar.make(viewGroup, getResources().getString(R.string.offline), Snackbar.LENGTH_LONG).show();
                }
                handler.postDelayed(this, 2000);
            }
        });

        //First start
        this.switchFragment(new HomeFragment(), false);

        //Add to Activity
        final SharedPreferences prefs = ((App)getApplication()).getPrefs();
        FirebaseMessaging.getInstance().subscribeToTopic("pushNotifications_"+prefs.getString("userId", ""));
    }

    public int getLayoutId(){
        return R.layout.activity_main;
    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        item.setChecked(true);

        switch (item.getItemId()) {
            case R.id.menu_home:
                switchFragment(new HomeFragment(), false);
                break;
            case R.id.menu_nuevo_evento:
                final Bundle bundle = new Bundle();
                bundle.putBoolean("navigation", true);
                final NuevoPlanFragment fragment = new NuevoPlanFragment();
                fragment.setArguments(bundle);
                switchFragment(fragment, false);
                break;
            case R.id.menu_notificaciones:
                switchFragment(new NotificacionesFragment(), false);
                break;
            case R.id.menu_red:
                switchFragment(new RedFragment(), false);
                break;
            case R.id.menu_perfil:
                switchFragment(new PerfilFragment(), false);
                break;
            case R.id.menu_cerrar:
                openPerfilDialog();
                break;
            default:
                break;
        }
        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    private void updateUI(){
        final DaoSession greendaoDb = ((App)getApplication()).getDaoSession();
        final SharedPreferences prefs = ((App)getApplication()).getPrefs();
        final String userId = prefs.getString("userId", "");
        final Usuario usuario = greendaoDb.getUsuarioDao().load(userId);

        if(usuario.getPhoto() != null){
            if(usuario.getLoginType() != null
                    && usuario.getLoginType() == TipoLoginUserEnum.FACEBOOK.getId()) {
                Picasso.with(getApplicationContext()).load(usuario.getPhoto()).into(userImage);
            }else{
                final Bitmap bitmap = ImageUtils.convert(usuario.getPhoto());
                userImage.setImageBitmap(bitmap);
            }
        }
        userName.setText(usuario.getName());
    }

    public void switchFragment(Fragment fragment, boolean customAnimation) {
        final FragmentTransaction fragmentTransaction = getSupportFragmentManager().beginTransaction();
        if(customAnimation){
            fragmentTransaction.setCustomAnimations(R.anim.enter_from_left, R.anim.exit_to_right);
        }else{
            fragmentTransaction.setCustomAnimations(android.R.anim.fade_in, android.R.anim.fade_out);
        }
        fragmentTransaction.replace(R.id.frame_container, fragment);
        fragmentTransaction.commit();
    }

    public void switchFragment(Fragment fragment) {
        final FragmentTransaction fragmentTransaction = getSupportFragmentManager().beginTransaction();
        fragmentTransaction.setCustomAnimations(R.anim.transition_from_up, R.anim.transition_to_down);
        fragmentTransaction.replace(R.id.frame_container, fragment).commit();
    }

    public void switchFragmentToUp(Fragment fragment) {
        final FragmentTransaction fragmentTransaction = getSupportFragmentManager().beginTransaction();
        fragmentTransaction.setCustomAnimations(R.anim.transition_up, R.anim.transition_down);
        fragmentTransaction.replace(R.id.frame_container, fragment).commit();
    }

    private void openPerfilDialog(){
        AlertDialog.Builder builder;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            builder = new AlertDialog.Builder(MainActivity.this, android.R.style.Theme_Material_Dialog_Alert);
        } else {
            builder = new AlertDialog.Builder(MainActivity.this);
        }
        builder.setTitle(getResources().getString(R.string.eliminar_cuenta))
                .setMessage(getResources().getString(R.string.eliminar_cuenta_info))
                .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        progressBarDialog.show();

                        FirebaseMessaging.getInstance().unsubscribeFromTopic("pushNotifications_"+prefs.getString("userId", ""));

                        //Cerrar sesión
                        //Se borra la base de datos local
                        final Database db = ((App) getApplication()).getDb();
                        DaoMaster.dropAllTables(db,true);
                        DaoMaster.createAllTables(db, true);

                        //Se borran las preferencias
                        final SharedPreferences.Editor editor = prefs.edit();
                        editor.clear();
                        editor.commit();

                        FirebaseAuth.getInstance().signOut();
                        LoginManager.getInstance().logOut();

                        //Se vuelve a la pantalla de Registro
                        finish();
                        final Intent intent = new Intent(MainActivity.this, RegistroActivity.class);
                        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
                        startActivity(intent);
                        progressBarDialog.hide();
                    }
                })
                .setNegativeButton(getResources().getString(R.string.cancelar), new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        //Do not anything
                    }
                })
                .setNeutralButton(getResources().getString(R.string.dar_baja), new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        progressBarDialog.show();
                        //Eliminar
                        final FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
                        if(user != null){
                            final String uid = user.getUid();
                            Usuario usuario = daoSession.getUsuarioDao().load(uid);

                            FirebaseMessaging.getInstance()
                                    .unsubscribeFromTopic("pushNotifications_"+prefs.getString("userId", ""));

                            //Se borran las preferencias
                            final SharedPreferences.Editor editor = prefs.edit();
                            editor.clear();
                            editor.commit();

                            //Se borra el usuario de Firebase
                            realtimeDB.child(FirebaseDBEnum.USUARIOS.getType()).child(uid).removeValue();

                            final List<Comentario> listaComentarios = daoSession.getComentarioDao().loadAll();
                            for(Comentario comentario : listaComentarios){
                                realtimeDB.child(FirebaseDBEnum.COMENTARIOS.getType()).child(comentario.getPlanId()).child(comentario.getId()).removeValue();
                            }

                            //Se borra lo relacionado con el usuario: planes, eventos, notificaciones...
                            final List<Plan> listaPlanes = daoSession.getPlanDao().loadAll();
                            for(Plan plan : listaPlanes){
                                if (TiposPlanes.EVENTOS.getId() == plan.getPlanType()){
                                    realtimeDB.child(FirebaseDBEnum.EVENTOS.getType()).child(plan.getId()).removeValue();
                                    realtimeDB.child(FirebaseDBEnum.EVENTOS_LOCATION.getType()).child(plan.getId()).removeValue();
                                }else{
                                    realtimeDB.child(FirebaseDBEnum.GRUPOS.getType()).child(plan.getId()).removeValue();
                                    realtimeDB.child(FirebaseDBEnum.GRUPOS_LOCATION.getType()).child(plan.getId()).removeValue();
                                }
                                realtimeDB.child(FirebaseDBEnum.COMENTARIOS.getType()).child(plan.getId()).removeValue();
                            }

                            final Query query = realtimeDB.child(FirebaseDBEnum.NOTIFICACIONES.getType()).orderByChild("name").equalTo(usuario.getId());
                            query.addListenerForSingleValueEvent(new ValueEventListener() {
                                @Override
                                public void onDataChange(DataSnapshot dataSnapshot) {
                                    for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {
                                        postSnapshot.getRef().removeValue();
                                    }
                                }

                                @Override
                                public void onCancelled(DatabaseError databaseError) {
                                    System.out.println("onCancelled");
                                }
                            });

                            //Se borra la base de datos local
                            final Database db = ((App) getApplication()).getDb();
                            DaoMaster.dropAllTables(db,true);
                            DaoMaster.createAllTables(db, true);

                            AuthCredential credential;
                            if(usuario.getLoginType() != null
                                    && usuario.getLoginType() == 1){
                                credential = FacebookAuthProvider.getCredential(AccessToken.getCurrentAccessToken().getToken());
                            }else{
                                credential = EmailAuthProvider.getCredential(user.getEmail(), "password1234");
                            }
                            // Prompt the user to re-provide their sign-in credentials
                            user.reauthenticate(credential)
                                    .addOnCompleteListener(new OnCompleteListener<Void>() {
                                        @Override
                                        public void onComplete(@NonNull Task<Void> task) {
                                            Log.d(TAG, "User re-authenticated.");
                                            //Se vuelve a la pantalla de Registro
                                            user.delete()
                                                    .addOnCompleteListener(new OnCompleteListener<Void>() {
                                                        @Override
                                                        public void onComplete(@NonNull Task<Void> task) {
                                                            if (task.isSuccessful()) {
                                                                Log.d(TAG, "User account deleted.");
                                                            }

                                                            //Se vuelve a la pantalla de Registro
                                                            finish();
                                                            final Intent intent = new Intent(MainActivity.this, RegistroActivity.class);
                                                            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
                                                            startActivity(intent);

                                                            FirebaseAuth.getInstance().signOut();
                                                            LoginManager.getInstance().logOut();
                                                            progressBarDialog.hide();
                                                        }
                                                    });
                                        }
                                    });

                        }else{
                            progressBarDialog.hide();
                        }
                    }
                })
                .setIcon(android.R.drawable.ic_dialog_alert)
                .show();
    }

}
