package edu.uoc.android.socialplan.activities.planes;

import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.location.Address;
import android.location.Geocoder;
import android.os.Bundle;
import android.support.v4.content.res.ResourcesCompat;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.SearchView;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.application.BaseActivity;
import edu.uoc.android.socialplan.utils.GeneralUtils;

/**
 * Created by alejandroresuagarcia on 18/11/17.
 */
public class PlacesSearchActivity extends BaseActivity {

    private static final String TAG = "PlacesSearchActivity";

    @BindView(R.id.search_view) SearchView searchView;
    @BindView(R.id.list_view) ListView listView;
    @BindView(R.id.toolbar) public Toolbar toolbar;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setSupportActionBar(toolbar);

        Drawable drawable = ResourcesCompat.getDrawable(getResources(), android.R.drawable.ic_menu_close_clear_cancel, null);
        drawable = DrawableCompat.wrap(drawable);
        DrawableCompat.setTint(drawable, Color.WHITE);

        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
                final Geocoder geo = new Geocoder(getApplicationContext());
                try {
                    final List<Address> lista = geo.getFromLocationName(query, 100);
                    Log.d(TAG, String.valueOf(lista.size()));
                    setAdapter(lista);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                return true;
            }

            @Override
            public boolean onQueryTextChange(String newText) {
                return true;
            }

            private void setAdapter(final List<Address> lista) {

                final List<String> listaDirecciones = new ArrayList<String>();
                for(Address a: lista){
                    if(GeneralUtils.coordinatesValid(a.getLongitude(), a.getLatitude())){
                        listaDirecciones.add(a.getFeatureName());
                    }
                }

                String[] values = new String[listaDirecciones.size()];
                values = listaDirecciones.toArray(values);

                final ArrayAdapter<String> adapter = new ArrayAdapter<String>(getApplicationContext(),
                        android.R.layout.simple_list_item_1,
                        android.R.id.text1, values);

                listView.setAdapter(adapter);

                listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                    @Override
                    public void onItemClick(AdapterView<?> parent, View view,
                                            int position, long id) {
                        Address a = lista.get(position);

                        Intent resultIntent = new Intent();
                        resultIntent.putExtra("lugar", a.getFeatureName());
                        resultIntent.putExtra("latitud", a.getLatitude());
                        resultIntent.putExtra("longitud", a.getLongitude());
                        setResult(PlacesSearchActivity.RESULT_OK, resultIntent);
                        finish();
                    }
                });
            }
        });
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == android.R.id.home) {
            // handle close button click here
            finish();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    public int getLayoutId() {
        return R.layout.activity_buscar_lugares;
    }

}