package edu.uoc.android.socialplan.activities.registro;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.support.design.widget.TextInputEditText;
import android.support.design.widget.TextInputLayout;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.EmailAuthProvider;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import butterknife.BindView;
import butterknife.OnClick;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.application.BaseActivity;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.models.Plan;
import edu.uoc.android.socialplan.models.Usuario;
import edu.uoc.android.socialplan.utils.GeneralUtils;
import edu.uoc.android.socialplan.utils.NetworkUtils;
import edu.uoc.android.socialplan.utils.ValidacionUtils;
import edu.uoc.android.socialplan.views.CustomProgressBarDialog;

/**
 * Created by alejandroresuagarcia on 6/11/17.
 */

public class LoginActivity extends BaseActivity {

    @BindView(R.id.user_mail) EditText userMail;
    @BindView(R.id.user_password) EditText userPassword;

    private FirebaseAuth mAuth;
    private DatabaseReference realtimeDB;
    private CustomProgressBarDialog progressBarDialog;

    private static String TAG = "LoginActivity";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mAuth = FirebaseAuth.getInstance();
        realtimeDB = ((App)getApplication()).getRealtimeDB();
        this.progressBarDialog = GeneralUtils.getCustomProgressBarDialog(LoginActivity.this);

        userMail.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                userMail.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        userPassword.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                userPassword.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
    }

    @Override
    public int getLayoutId() {
        return R.layout.activity_login;
    }

    @OnClick(R.id.login)
    public void loginAction(View view) {
        if(NetworkUtils.isNetworkConnected(LoginActivity.this)){
            this.signIn(userMail.getText().toString(), userPassword.getText().toString(), view);
        }else{
            Snackbar.make(view, getResources().getString(R.string.offline), Snackbar.LENGTH_LONG).show();
        }
    }

    private void signIn(final String email, final String password, final View view) {
        progressBarDialog.show();
        boolean error = false;
        if(!ValidacionUtils.isValidEmail(email)){
            ValidacionUtils.hayErrorEditText(getResources(), userMail);
            error = true;
        }
        if(!ValidacionUtils.isRequired(password)){
            ValidacionUtils.hayErrorEditText(getResources(), userPassword);
            error = true;
        }
        if(!error){
            mAuth.signInWithEmailAndPassword(email, password)
                    .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                        @Override
                        public void onComplete(@NonNull Task<AuthResult> task) {
                            if (task.isSuccessful()) {
                                // Sign in success, update UI with the signed-in user's information
                                Log.d(TAG, "signInWithEmail:success");
                                FirebaseUser user = mAuth.getCurrentUser();

                                getUserFromFirebase(user);
                            } else {
                                // If sign in fails, display a message to the user.
                                Log.w(TAG, "signInWithEmail:failure", task.getException());
                                Snackbar.make(view, getResources().getString(R.string.credenciales), Snackbar.LENGTH_LONG).show();
                                progressBarDialog.hide();
                            }
                        }
                    }).addOnFailureListener(this, new OnFailureListener() {
                @Override
                public void onFailure(@NonNull Exception e) {
                    // If sign in fails, display a message to the user.
                    Snackbar.make(view, getResources().getString(R.string.credenciales), Snackbar.LENGTH_LONG).show();
                    progressBarDialog.hide();
                }
            });
        }else{
            Snackbar.make(view, getResources().getString(R.string.validacion_error), Snackbar.LENGTH_LONG).show();
            progressBarDialog.hide();
        }
    }

    private void addUserGreenDao(Usuario usuario) {
        //Se guarda el ID en las preferencias
        SharedPreferences.Editor editor = getPrefs().edit();
        editor.putString("userId", usuario.getId());
        editor.apply();

        //Se guarda el usuario
        getGreendaoDb().getUsuarioDao().insertOrReplace(usuario);

        //Se recupera toda la información de firebase para el usuario y se guarda en local
        final Query query = realtimeDB.child(FirebaseDBEnum.EVENTOS.getType()).orderByChild("userId").equalTo(usuario.getId());;
        query.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                final GenericTypeIndicator<HashMap<String, Plan>> generic = new GenericTypeIndicator<HashMap<String, Plan>>() {};
                HashMap<String, Plan> map = dataSnapshot.getValue(generic);
                if (map != null && !map.isEmpty()) {
                    for(Plan plan : map.values()){
                        getGreendaoDb().getPlanDao().insertOrReplace(plan);
                    }
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
            }
        });

        final Query queryGrupos = realtimeDB.child(FirebaseDBEnum.GRUPOS.getType()).orderByChild("userId").equalTo(usuario.getId());;
        queryGrupos.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                final GenericTypeIndicator<HashMap<String, Plan>> generic = new GenericTypeIndicator<HashMap<String, Plan>>() {};
                HashMap<String, Plan> map = dataSnapshot.getValue(generic);
                if (map != null && !map.isEmpty()) {
                    for(Plan plan : map.values()){
                        getGreendaoDb().getPlanDao().insertOrReplace(plan);
                    }
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
            }
        });
        progressBarDialog.hide();
        final Intent intent = new Intent(LoginActivity.this, MainActivity.class);
        startActivity(intent);
        finish();
    }

    private void getUserFromFirebase(FirebaseUser user) {
        Query query = realtimeDB.child(FirebaseDBEnum.USUARIOS.getType()).child(user.getUid());
        query.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                final Usuario usuario = dataSnapshot.getValue(Usuario.class);
                addUserGreenDao(usuario);
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
                progressBarDialog.hide();
            }
        });
    }

    @OnClick(R.id.volver_button)
    public void volverAction(View view) {
        final Intent intent = new Intent(this, RegistroActivity.class);
        startActivity(intent);
        finish();
    }

    @OnClick(R.id.forget_button)
    public void forgetAction(final View view) {
        progressBarDialog.show();
        if(NetworkUtils.isNetworkConnected(LoginActivity.this)){
            boolean error = false;
            if(!ValidacionUtils.isValidEmail(userMail.getText().toString())){
                ValidacionUtils.hayErrorEditText(getResources(), userMail);
                error = true;
            }

            if(!error){
                FirebaseAuth auth = FirebaseAuth.getInstance();
                auth.sendPasswordResetEmail(userMail.getText().toString()).addOnCompleteListener(new OnCompleteListener<Void>() {
                    @Override
                    public void onComplete(@NonNull Task<Void> task) {
                        if (task.isSuccessful()) {
                            Log.d(TAG, "Email sent.");
                            Snackbar.make(view, getResources().getString(R.string.compruebe_correo), Snackbar.LENGTH_LONG).show();
                        }
                        progressBarDialog.hide();
                    }
                });
            }else{
                Snackbar.make(view, getResources().getString(R.string.validacion_error), Snackbar.LENGTH_LONG).show();
                progressBarDialog.hide();
            }
        }else{
            progressBarDialog.hide();
            Snackbar.make(view, getResources().getString(R.string.offline), Snackbar.LENGTH_LONG).show();
        }


    }
}
