package edu.uoc.android.socialplan.activities.registro;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import com.facebook.AccessToken;
import com.facebook.CallbackManager;
import com.facebook.FacebookCallback;
import com.facebook.FacebookException;
import com.facebook.login.LoginResult;
import com.facebook.login.widget.LoginButton;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FacebookAuthProvider;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import butterknife.BindView;
import butterknife.OnClick;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.application.BaseActivity;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.constants.TipoLoginUserEnum;
import edu.uoc.android.socialplan.models.Usuario;
import edu.uoc.android.socialplan.utils.GeneralUtils;
import edu.uoc.android.socialplan.utils.NetworkUtils;
import edu.uoc.android.socialplan.views.CustomProgressBarDialog;

/**
 * Created by alejandroresuagarcia on 6/11/17.
 */

public class RegistroActivity extends BaseActivity {

    private static String TAG = "RegistroActivity";

    @BindView(R.id.facebookButton) LoginButton loginButton;

    private CallbackManager mCallbackManager;
    private CustomProgressBarDialog progressBarDialog;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        this.progressBarDialog = GeneralUtils.getCustomProgressBarDialog(RegistroActivity.this);
        mCallbackManager = CallbackManager.Factory.create();
    }

    @Override
    public int getLayoutId() {
        return R.layout.activity_registro;
    }

    @OnClick(R.id.iniciarSesionButton)
    public void loginAction(View view) {
        progressBarDialog.show();
        final Intent intent = new Intent(this, LoginActivity.class);
        startActivity(intent);
        finish();
        progressBarDialog.hide();
    }

    @OnClick(R.id.facebookButton)
    public void facebookAction(View view) {
        progressBarDialog.show();
        if(NetworkUtils.isNetworkConnected(RegistroActivity.this)){
            // Initialize Facebook Login button
            loginButton.setReadPermissions("email", "public_profile");
            loginButton.registerCallback(mCallbackManager, new FacebookCallback<LoginResult>() {
                @Override
                public void onSuccess(LoginResult loginResult) {
                    Log.d(TAG, "facebook:onSuccess:" + loginResult);
                    handleFacebookAccessToken(loginResult.getAccessToken());
                    progressBarDialog.hide();
                }

                @Override
                public void onCancel() {
                    Log.d(TAG, "facebook:onCancel");
                    progressBarDialog.hide();
                }

                @Override
                public void onError(FacebookException error) {
                    Log.d(TAG, "facebook:onError", error);
                    progressBarDialog.hide();
                }
            });
        }else{
            progressBarDialog.hide();
            Snackbar.make(view, getResources().getString(R.string.offline), Snackbar.LENGTH_LONG).show();
        }
    }

    @OnClick(R.id.registroButton)
    public void registerAction(View view) {
        progressBarDialog.show();
        final Intent intent = new Intent(this, RegistroFormActivity.class);
        startActivity(intent);
        finish();
        progressBarDialog.hide();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        // Pass the activity result back to the Facebook SDK
        mCallbackManager.onActivityResult(requestCode, resultCode, data);
    }

    private void handleFacebookAccessToken(AccessToken token) {
        progressBarDialog.show();

        Log.d(TAG, "handleFacebookAccessToken:" + token);

        AuthCredential credential = FacebookAuthProvider.getCredential(token.getToken());
        getmAuth().signInWithCredential(credential)
                .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            // Sign in success, update UI with the signed-in user's information
                            Log.d(TAG, "signInWithCredential:success");
                            FirebaseUser user = getmAuth().getCurrentUser();
                            addUser(user);
                        } else {
                            // If sign in fails, display a message to the user.
                            Log.w(TAG, "signInWithCredential:failure", task.getException());
                            Toast.makeText(RegistroActivity.this, getResources().getString(R.string.error),
                                    Toast.LENGTH_SHORT).show();
                            progressBarDialog.hide();
                        }

                    }
                });
    }

    private void addUser(final FirebaseUser user){
        Query query = getRealtimeDB().child(FirebaseDBEnum.USUARIOS.getType()).child(user.getUid());
        query.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if(dataSnapshot.exists()){
                    final Usuario usuario = dataSnapshot.getValue(Usuario.class);
                    addUserGreenDao(usuario);
                }else{
                    final Usuario usuario = new Usuario();
                    usuario.setId(user.getUid());
                    usuario.setMail(user.getEmail());
                    usuario.setLoginType(TipoLoginUserEnum.FACEBOOK.getId());
                    usuario.setPhoto(user.getPhotoUrl() != null ? user.getPhotoUrl().toString() : "");
                    usuario.setName(user.getDisplayName());

                    // Añadir nuevo usuario
                    getRealtimeDB().child(FirebaseDBEnum.USUARIOS.getType()).child(user.getUid()).setValue(usuario);
                    addUserGreenDao(usuario);
                }
                final Intent intent = new Intent(RegistroActivity.this, MainActivity.class);
                startActivity(intent);
                finish();
                progressBarDialog.hide();
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
                progressBarDialog.hide();
            }
        });
    }

    private void addUserGreenDao(Usuario usuario) {
        //Se guarda el ID en las preferencias
        SharedPreferences.Editor editor = getPrefs().edit();
        editor.putString("userId", usuario.getId());
        editor.apply();

        //Se guarda el usuario
        getGreendaoDb().getUsuarioDao().insertOrReplace(usuario);
    }
}
