package edu.uoc.android.socialplan.activities.registro;

import android.Manifest;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.location.Location;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Toast;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.ValueEventListener;

import java.io.IOException;
import java.io.InputStream;

import butterknife.BindView;
import butterknife.OnClick;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.application.BaseActivity;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.models.DaoSession;
import edu.uoc.android.socialplan.models.Usuario;
import edu.uoc.android.socialplan.utils.GeneralUtils;
import edu.uoc.android.socialplan.utils.ImageUtils;
import edu.uoc.android.socialplan.utils.NetworkUtils;
import edu.uoc.android.socialplan.utils.ValidacionUtils;
import edu.uoc.android.socialplan.views.CustomProgressBarDialog;

/**
 * Created by alejandroresuagarcia on 6/11/17.
 */

public class RegistroFormActivity extends BaseActivity {

    @BindView(R.id.user_mail) EditText userMail;
    @BindView(R.id.user_password) EditText userPassword;
    @BindView(R.id.user_password_repeat) EditText userPasswordRepeat;
    @BindView(R.id.user_image) ImageView userImage;
    @BindView(R.id.user_name) EditText userName;

    private FirebaseAuth mAuth;
    private DaoSession greendaoDb;
    private SharedPreferences prefs;
    private DatabaseReference realtimeDB;
    private static String TAG = "RegistroFormActivity";

    private int GALLERY = 1, CAMERA = 2;
    private static final int MY_CAMERA_REQUEST_CODE = 100;
    private CustomProgressBarDialog progressBarDialog;

    private String userImageB64;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mAuth = FirebaseAuth.getInstance();
        greendaoDb = ((App)getApplication()).getDaoSession();
        prefs = ((App)getApplication()).getPrefs();
        realtimeDB = ((App)getApplication()).getRealtimeDB();
        this.progressBarDialog = GeneralUtils.getCustomProgressBarDialog(RegistroFormActivity.this);

        userName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                userName.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        userMail.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                userMail.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        userPassword.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                userPassword.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        userPasswordRepeat.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                userPasswordRepeat.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
    }

    @Override
    public int getLayoutId() {
        return R.layout.activity_registro_form;
    }

    @OnClick(R.id.user_image)
    public void userImageButton(View view) {
        AlertDialog.Builder pictureDialog = new AlertDialog.Builder(this);
        pictureDialog.setTitle(getResources().getString(R.string.seleccionar_opcion));
        String[] pictureDialogItems = {
                getResources().getString(R.string.seleccionar_foto),
                getResources().getString(R.string.camara)};
        pictureDialog.setItems(pictureDialogItems, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        switch (which) {
                            case 0:
                                choosePhotoFromGallery();
                                break;
                            case 1:
                                takePhotoFromCamera();
                                break;
                        }
                    }
                });
        pictureDialog.show();
    }

    public void choosePhotoFromGallery() {
        final Intent photoPickerIntent = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
        startActivityForResult(photoPickerIntent, GALLERY);
    }

    private void takePhotoFromCamera() {
        if (ContextCompat.checkSelfPermission(RegistroFormActivity.this, Manifest.permission.CAMERA)
                != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, new String[] {Manifest.permission.CAMERA}, MY_CAMERA_REQUEST_CODE);
        }else{
            final Intent intent = new Intent(android.provider.MediaStore.ACTION_IMAGE_CAPTURE);
            startActivityForResult(intent, CAMERA);

        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case MY_CAMERA_REQUEST_CODE: {
                if (ContextCompat.checkSelfPermission(RegistroFormActivity.this, Manifest.permission.CAMERA)
                        == PackageManager.PERMISSION_GRANTED) {
                    final Intent intent = new Intent(android.provider.MediaStore.ACTION_IMAGE_CAPTURE);
                    startActivityForResult(intent, CAMERA);
                } else {
                    Log.d(TAG, "Deny permission");
                }
            }
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {

        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == RESULT_CANCELED) {
            return;
        }
        if (requestCode == GALLERY) {
            if (data != null) {
                Uri contentURI = data.getData();
                try {
                    InputStream image_stream = getContentResolver().openInputStream(contentURI);
                    Bitmap bitmap = BitmapFactory.decodeStream(image_stream);
                    bitmap=Bitmap.createScaledBitmap(bitmap, 200, 200, true);
                    userImage.setImageBitmap(bitmap);
                    userImageB64 = ImageUtils.convert(bitmap);
                } catch (IOException e) {
                    e.printStackTrace();
                    Toast.makeText(RegistroFormActivity.this, getResources().getString(R.string.error), Toast.LENGTH_SHORT).show();
                }
            }
        } else if (requestCode == CAMERA) {
            Bitmap thumbnail = (Bitmap) data.getExtras().get("data");
            userImage.setImageBitmap(thumbnail);
            userImageB64 = ImageUtils.convert(thumbnail);
        }
    }

    @OnClick(R.id.crear_usuario)
    public void crearUsuarioAction(View view) {
        progressBarDialog.show();
        if(NetworkUtils.isNetworkConnected(RegistroFormActivity.this)){
            boolean error = false;
            boolean errorPass = false;
            if(!ValidacionUtils.isRequired(userName.getText().toString())){
                ValidacionUtils.hayErrorEditText(getResources(), userName);
                error = true;
            }
            if(!ValidacionUtils.isValidEmail(userMail.getText().toString())){
                ValidacionUtils.hayErrorEditText(getResources(), userMail);
                error = true;
            }
            if(!ValidacionUtils.isRequired(userPassword.getText().toString())) {
                ValidacionUtils.hayErrorEditText(getResources(), userPassword);
                error = true;
            }
            if(!ValidacionUtils.isRequired(userPasswordRepeat.getText().toString())) {
                ValidacionUtils.hayErrorEditText(getResources(), userPasswordRepeat);
                error = true;
            }else if(!ValidacionUtils.isEqual(userPassword.getText().toString(), userPasswordRepeat.getText().toString())){
                ValidacionUtils.hayErrorEditText(getResources(), userPassword);
                ValidacionUtils.hayErrorEditText(getResources(), userPasswordRepeat);
                errorPass = true;
                Snackbar.make(view, getResources().getString(R.string.caracteres_pass_repeat), Snackbar.LENGTH_LONG).show();
                error = true;
            }else if(userPassword.getText().toString().length()<6){
                errorPass = true;
                Snackbar.make(view, getResources().getString(R.string.caracteres_pass), Snackbar.LENGTH_LONG).show();
                error = true;
            }
            if(!error){
                mAuth.createUserWithEmailAndPassword(userMail.getText().toString(), userPassword.getText().toString())
                        .addOnCompleteListener(RegistroFormActivity.this, new OnCompleteListener<AuthResult>() {
                            @Override
                            public void onComplete(@NonNull Task<AuthResult> task) {
                                if (task.isSuccessful()) {
                                    // Sign in success, update UI with the signed-in user's information
                                    Log.d(TAG, "createUserWithEmail:success");
                                    FirebaseUser user = mAuth.getCurrentUser();
                                    addUser(user);
                                }else if(task.getException() != null
                                        && task.getException().getMessage() != null){
                                    Toast.makeText(RegistroFormActivity.this, task.getException().getMessage(),
                                            Toast.LENGTH_SHORT).show();
                                    progressBarDialog.hide();
                                }else {
                                    // If sign in fails, display a message to the user.
                                    Log.w(TAG, "createUserWithEmail:failure", task.getException());
                                    Toast.makeText(RegistroFormActivity.this, getResources().getString(R.string.error),
                                            Toast.LENGTH_SHORT).show();
                                    progressBarDialog.hide();
                                }
                            }
                        });
            }else{
                if(!errorPass){
                    Snackbar.make(view, getResources().getString(R.string.validacion_error), Snackbar.LENGTH_LONG).show();
                }
                progressBarDialog.hide();
            }
        }else{
            progressBarDialog.hide();
            Snackbar.make(view, getResources().getString(R.string.offline), Snackbar.LENGTH_LONG).show();
        }
    }

    private void addUser(final FirebaseUser user){
        final Usuario usuario = new Usuario();
        usuario.setId(user.getUid());
        usuario.setMail(user.getEmail());
        if(userImageB64 != null){
            usuario.setPhoto(userImageB64);
        }
        usuario.setName(userName.getText().toString());

        // Añadir nuevo usuario
        final DatabaseReference reference = realtimeDB.child(FirebaseDBEnum.USUARIOS.getType()).child(user.getUid());
        reference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if(dataSnapshot.exists()){
                    reference.addListenerForSingleValueEvent(new ValueEventListener() {
                        @Override
                        public void onDataChange(DataSnapshot dataSnapshot) {
                            final Usuario usuario = dataSnapshot.getValue(Usuario.class);
                            addUserGreenDao(usuario);
                        }

                        @Override
                        public void onCancelled(DatabaseError databaseError) {
                            System.out.println("onCancelled");
                        }
                    });
                }else{
                    realtimeDB.child(FirebaseDBEnum.USUARIOS.getType()).child(user.getUid()).setValue(usuario);
                    addUserGreenDao(usuario);
                    final Intent intent = new Intent(RegistroFormActivity.this, MainActivity.class);
                    startActivity(intent);
                    finish();
                }
                progressBarDialog.hide();
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
            }
        });
    }

    private void addUserGreenDao(Usuario usuario) {
        //Se guarda el ID en las preferencias
        SharedPreferences.Editor editor = prefs.edit();
        editor.putString("userId", usuario.getId());
        editor.apply();

        //Se guarda el usuario
        greendaoDb.getUsuarioDao().insertOrReplace(usuario);
    }

    @OnClick(R.id.volver_button)
    public void volverAction(View view) {
        final Intent intent = new Intent(this, RegistroActivity.class);
        startActivity(intent);
        finish();
    }

}
