package edu.uoc.android.socialplan.adapters;

import android.content.Context;
import android.graphics.Bitmap;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.constants.TipoLoginUserEnum;
import edu.uoc.android.socialplan.holders.ComentariosHolder;
import edu.uoc.android.socialplan.models.Comentario;
import edu.uoc.android.socialplan.utils.ImageUtils;

public class ComentariosRecyclerViewAdapter extends RecyclerView.Adapter<ComentariosHolder> {

    private List<Comentario> items;
    private Context mContext;

    private String userId;

    private static String TAG = "ComentariosRecyclerViewAdapter";

    public ComentariosRecyclerViewAdapter(Context context, List<Comentario> items) {
        this.mContext = context;
        this.items = items;
    }

    @Override
    public ComentariosHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_comentario, parent, false);
        return new ComentariosHolder(view);
    }

    @Override
    public void onBindViewHolder(final ComentariosHolder holder, int pos) {
        final Comentario comentario = items.get(pos);
        holder.getComent().setText(comentario.getComentario());
        holder.getUserComentName().setText(comentario.getUserName());

        if(comentario.getUserImage() != null){
            if(comentario.getLoginType() != null
                    && comentario.getLoginType() == TipoLoginUserEnum.FACEBOOK.getId()) {
                Picasso.with(mContext).load(comentario.getUserImage()).into(holder.getUserComentImage());
            }else{
                final Bitmap bitmap = ImageUtils.convert(comentario.getUserImage());
                holder.getUserComentImage().setImageBitmap(bitmap);
            }
        }

        holder.getUserComentDate().setText(getDate(comentario.getDate()));
    }

    private String getDate(Date date){
        final String myFormat = mContext.getResources().getString(R.string.date_mask); //In which you need put here
        SimpleDateFormat sdf = new SimpleDateFormat(myFormat, new Locale("es"));
        return sdf.format(date);
    }

    @Override
    public int getItemCount() {
        return items.size();
    }

    public void addItem(List<Comentario> lista){
        this.items.clear();
        this.items.addAll(lista);
        notifyDataSetChanged();
    }

}
