package edu.uoc.android.socialplan.adapters;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.constants.TipoNotificacionesEnum;
import edu.uoc.android.socialplan.holders.NotificacionesHolder;
import edu.uoc.android.socialplan.models.Notificacion;

public class NotificacionesRecyclerViewAdapter extends RecyclerView.Adapter<NotificacionesHolder> {

    private List<Notificacion> items;
    private Context mContext;

    private static String TAG = "NotificacionesRecyclerViewAdapter";

    public NotificacionesRecyclerViewAdapter(Context context, List<Notificacion> items) {
        this.mContext = context;
        this.items = items;
    }

    @Override
    public NotificacionesHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_notificacion, parent, false);
        return new NotificacionesHolder(view);
    }

    @Override
    public void onBindViewHolder(final NotificacionesHolder holder, int pos) {
        final Notificacion notificacion = items.get(pos);
        holder.getComent().setText(notificacion.getText());
        if(TipoNotificacionesEnum.LIKE.getId() == notificacion.getType()){
            holder.getmView().setBackgroundColor(mContext.getResources().getColor(R.color.colorComent3Light));
        }else if(TipoNotificacionesEnum.DELETE.getId() == notificacion.getType()){
            holder.getmView().setBackgroundColor(mContext.getResources().getColor(R.color.colorPrimaryLight));
        }else if(TipoNotificacionesEnum.JOIN.getId() == notificacion.getType()){
            holder.getmView().setBackgroundColor(mContext.getResources().getColor(R.color.colorComent1Light));
        }else{
            holder.getmView().setBackgroundColor(mContext.getResources().getColor(R.color.colorComent2Light));
        }

        holder.getDateTv().setText(getDate(notificacion.getDate()));
    }

    private String getDate(Date date){
        final String myFormat = mContext.getResources().getString(R.string.date_mask); //In which you need put here
        SimpleDateFormat sdf = new SimpleDateFormat(myFormat, new Locale("es"));
        return sdf.format(date);
    }

    @Override
    public int getItemCount() {
        return items.size();
    }

    public void addItem(List<Notificacion> lista){
        this.items.clear();
        this.items.addAll(lista);
        notifyDataSetChanged();
    }

    public void clearItems(){
        this.items.clear();
        notifyDataSetChanged();
    }

}
