package edu.uoc.android.socialplan.adapters;

import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.like.LikeButton;
import com.like.OnLikeListener;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.constants.TipoNotificacionesEnum;
import edu.uoc.android.socialplan.constants.TiposPlanes;
import edu.uoc.android.socialplan.fragments.DetallePlanFragment;
import edu.uoc.android.socialplan.holders.PlanHolder;
import edu.uoc.android.socialplan.models.DaoSession;
import edu.uoc.android.socialplan.models.Plan;
import edu.uoc.android.socialplan.models.Usuario;
import edu.uoc.android.socialplan.utils.ImageUtils;
import edu.uoc.android.socialplan.utils.NotificationUtils;

public class PlanesRecyclerViewAdapter extends RecyclerView.Adapter<PlanHolder> {

    private List<Plan> items;
    private Context mContext;
    private SharedPreferences prefs;
    private DatabaseReference realtimeDB;
    private DaoSession daoSession;
    private String type;
    private String typeLocation;
    private int parentFragment;

    private String userId;
    private Usuario usuario;

    private static String TAG = "PlanesRecyclerViewAdapter";

    public PlanesRecyclerViewAdapter(Context context, List<Plan> items, String type, String typeLocation, int parentFragment) {
        this.mContext = context;
        this.items = items;
        this.type = type;
        this.typeLocation = typeLocation;
        this.parentFragment = parentFragment;
    }

    @Override
    public PlanHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_plan, parent, false);
        MainActivity mainActivity = (MainActivity) mContext;
        prefs = ((App)mainActivity.getApplication()).getPrefs();
        realtimeDB = ((App)mainActivity.getApplication()).getRealtimeDB();
        daoSession = ((App)mainActivity.getApplication()).getDaoSession();

        userId = prefs.getString("userId", "");
        usuario = daoSession.getUsuarioDao().load(userId);

        return new PlanHolder(view);
    }

    @Override
    public void onBindViewHolder(final PlanHolder holder, int pos) {
        final Plan plan = items.get(pos);

        holder.getEventoTitle().setText(plan.getTitle());
        if(plan.getImage() != null){
            final Bitmap bitmap = ImageUtils.convert(plan.getImage());
            holder.getEventoImage().setImageBitmap(bitmap);
        }

        holder.getPlanDate().setText(this.getDate(plan.getDate()));

        if(FirebaseDBEnum.EVENTOS.getType().equals(type)){
            holder.getLikesCountEvento().setVisibility(View.VISIBLE);
            holder.getLikesCountEventoLayout().setVisibility(View.VISIBLE);
            holder.getLikesLayout().setVisibility(View.VISIBLE);
            if(plan.getUsersCount() > 0){
                holder.getLikesCountEvento().setText(String.valueOf(plan.getUsersCount()));
            }
            if(userId.equals(plan.getUserId())){
                holder.getLikesLayout().setVisibility(View.GONE);
            }
            final DatabaseReference reference = realtimeDB.child(type).child(plan.getId()).child(FirebaseDBEnum.USUARIOS.getType());
            Query query = reference.child(userId);
            query.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    if(dataSnapshot.exists()){
                        holder.getLikeButton().setLiked(true);
                    }else{
                        holder.getLikeButton().setLiked(false);
                    }

                    holder.getLikeButton().setOnLikeListener(new OnLikeListener() {
                        @Override
                        public void liked(LikeButton likeButton) {
                            //Se actualizan los usuarios
                            Map<String, Object> childUpdates = new HashMap<>();
                            childUpdates.put(userId, true);
                            reference.updateChildren(childUpdates);
                            //Se actualizan los usuarios con likes
                            childUpdates = new HashMap<>();
                            int usersCount = plan.getUsersCount();
                            usersCount += 1;
                            childUpdates.put("usersCount", usersCount);
                            realtimeDB.child(type).child(plan.getId()).updateChildren(childUpdates);

                            holder.getLikesCountEvento().setText(String.valueOf(usersCount));
                            plan.setUsersCount(usersCount);

                            final String sb = mContext.getResources().getString(R.string.evento_like, usuario.getName(), holder.getEventoTitle().getText().toString());
                            NotificationUtils.sendNotificationToUser(plan.getUserId(), sb, TipoNotificacionesEnum.LIKE.getId(), realtimeDB, daoSession);
                        }

                        @Override
                        public void unLiked(LikeButton likeButton) {
                            //Se actualizan los usuarios
                            reference.child(userId).removeValue();
                            //Se actualizan los usuarios con likes
                            Map<String, Object> childUpdates = new HashMap<>();
                            int usersCount = plan.getUsersCount();
                            usersCount -= 1;
                            childUpdates.put("usersCount", usersCount);
                            realtimeDB.child(type).child(plan.getId()).updateChildren(childUpdates);

                            holder.getLikesCountEvento().setText(String.valueOf(usersCount));
                            plan.setUsersCount(usersCount);
                        }
                    });
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    System.out.println("onCancelled");
                }
            });
        }else{
            holder.getLikeButton().setVisibility(View.GONE);
            holder.getLikesCountEvento().setVisibility(View.VISIBLE);
            holder.getLikesCountEvento().setText(String.valueOf(plan.getUsersCount()));
            holder.getLikesCountEventoLayout().setVisibility(View.VISIBLE);

            if(plan.getUsersCount() == plan.getUsersMax()){
                holder.getCompleto().setVisibility(View.VISIBLE);
            }else{
                holder.getCompleto().setVisibility(View.GONE);
            }
        }

        holder.getmView().setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final DetallePlanFragment fragment = new DetallePlanFragment();
                Bundle bundle = new Bundle();
                bundle.putSerializable("plan", plan);
                bundle.putString("type", type);
                bundle.putString("typeLocation", typeLocation);
                bundle.putInt("parentFragment", parentFragment);
                fragment.setArguments(bundle);
                switchContent(fragment);
            }
        });
    }

    private String getDate(Date date){
        final String myFormat = mContext.getResources().getString(R.string.date_mask); //In which you need put here
        SimpleDateFormat sdf = new SimpleDateFormat(myFormat, new Locale("es"));
        return sdf.format(date);
    }

    @Override
    public int getItemCount() {
        return items.size();
    }

    public void addItem(Plan plan, String type){
        this.type = type;
        this.items.add(0, plan);
        notifyDataSetChanged();
    }

    public void clearItems(){
        this.items.clear();
        notifyDataSetChanged();
    }

    private void switchContent(Fragment fragment) {
        MainActivity mainActivity = (MainActivity) mContext;
        mainActivity.switchFragmentToUp(fragment);
    }

    public void removeItem(int position) {
        final Plan plan = items.get(position);

        //Borrar el id de greendao
        daoSession.getPlanDao().deleteByKey(plan.getId());

        //Borrar de firebase
        if(TiposPlanes.EVENTOS.getId() == plan.getPlanType()){
            realtimeDB.child(FirebaseDBEnum.EVENTOS.getType()).child(plan.getId()).removeValue();
            realtimeDB.child(FirebaseDBEnum.EVENTOS_LOCATION.getType()).child(plan.getId()).removeValue();
        }else{
            //Enviar notificación si es un grupo
            Query query = realtimeDB.child(FirebaseDBEnum.GRUPOS.getType()).child(plan.getId()).child(FirebaseDBEnum.USUARIOS.getType());
            query.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {
                        String userId = postSnapshot.getKey();

                        final String sb = mContext.getResources().getString(R.string.evento_delete, usuario.getName(), plan.getTitle());
                        NotificationUtils.sendNotificationToUser(userId, sb, TipoNotificacionesEnum.DELETE.getId(), realtimeDB, daoSession);
                    }
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    System.out.println("onCancelled");
                }
            });

            realtimeDB.child(FirebaseDBEnum.GRUPOS.getType()).child(plan.getId()).removeValue();
            realtimeDB.child(FirebaseDBEnum.GRUPOS_LOCATION.getType()).child(plan.getId()).removeValue();
        }

        //Borrar del listado y actualizar lista
        items.remove(position);
        notifyItemRemoved(position);
    }

}
