package edu.uoc.android.socialplan.fragments;

import android.Manifest;
import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TabHost;

import com.ethanhua.skeleton.Skeleton;
import com.ethanhua.skeleton.SkeletonScreen;
import com.firebase.geofire.GeoFire;
import com.firebase.geofire.GeoLocation;
import com.firebase.geofire.GeoQuery;
import com.firebase.geofire.GeoQueryEventListener;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.LocationSource;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.activities.planes.SearchActivity;
import edu.uoc.android.socialplan.adapters.PlanesRecyclerViewAdapter;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.constants.AdapterEnum;
import edu.uoc.android.socialplan.constants.TiposPlanes;
import edu.uoc.android.socialplan.models.DaoSession;
import edu.uoc.android.socialplan.models.Plan;
import edu.uoc.android.socialplan.models.Usuario;
import edu.uoc.android.socialplan.utils.GeneralUtils;
import edu.uoc.android.socialplan.utils.ImageUtils;

/**
 * Created by alejandroresuagarcia on 13/11/17.
 */

public class HomeFragment extends Fragment implements GoogleApiClient.ConnectionCallbacks,
        GoogleApiClient.OnConnectionFailedListener, LocationSource.OnLocationChangedListener,
        TabHost.OnTabChangeListener{

    @BindView(R.id.fab) FloatingActionButton fab;
    @BindView(R.id.eventos_recycler_view) RecyclerView eventosRV;
    @BindView(R.id.grupos_recycler_view) RecyclerView gruposRV;
    @BindView(R.id.tab_host) TabHost tabs;
    @BindView(R.id.swipe_eventos_container) SwipeRefreshLayout swipeEventosContainer;
    @BindView(R.id.swipe_grupos_container) SwipeRefreshLayout swipeGruposContainer;

    private static final int LOCATION_REQUEST_CODE = 1, SEARCH=2;
    private static final String TAG = "HomeFragment";

    private GoogleApiClient mGoogleApiClient;

    private SharedPreferences prefs;
    private DaoSession daoSession;
    private DatabaseReference realtimeDB;

    private PlanesRecyclerViewAdapter eventosAdapter;
    private PlanesRecyclerViewAdapter gruposAdapter;
    private SkeletonScreen skeletonScreen;

    private Map<String, Plan> map = new HashMap<String, Plan>();

    boolean estaBuscando = false;

    public HomeFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        final View rootView = inflater.inflate(R.layout.fragment_home, container, false);
        ButterKnife.bind(this, rootView);

        AppCompatActivity activity = (AppCompatActivity) getActivity();
        activity.setSupportActionBar(((MainActivity)getActivity()).toolbar);

        ((MainActivity)getActivity()).toggle = new ActionBarDrawerToggle(
                ((MainActivity)getActivity()), ((MainActivity)getActivity()).drawer, ((MainActivity)getActivity()).toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        ((MainActivity)getActivity()).drawer.addDrawerListener(((MainActivity)getActivity()).toggle);
        ((MainActivity)getActivity()).toggle.syncState();
        ((MainActivity)getActivity()).getSupportActionBar().setTitle(getResources().getString(R.string.home));

        setHasOptionsMenu(true);

        // Create an instance of GoogleAPIClient.
        if (this.mGoogleApiClient == null) {
            this.mGoogleApiClient = new GoogleApiClient.Builder(getContext())
                    .addConnectionCallbacks(this)
                    .addOnConnectionFailedListener(this)
                    .addApi(LocationServices.API)
                    .build();
        }

        mGoogleApiClient.connect();

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                switchContent(new NuevoPlanFragment());
            }
        });

        this.eventosAdapter = new PlanesRecyclerViewAdapter(getContext(), new ArrayList<Plan>(),
                FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType(), AdapterEnum.HOME_EVENTOS.getId());
        eventosRV.setAdapter(this.eventosAdapter);
        this.gruposAdapter = new PlanesRecyclerViewAdapter(getContext(), new ArrayList<Plan>(),
                FirebaseDBEnum.GRUPOS.getType(), FirebaseDBEnum.GRUPOS_LOCATION.getType(), AdapterEnum.HOME_GRUPOS.getId());
        gruposRV.setAdapter(this.gruposAdapter);

        prefs = ((App) getActivity().getApplication()).getPrefs();
        daoSession = ((App)getActivity().getApplication()).getDaoSession();
        realtimeDB = ((App) getActivity().getApplication()).getRealtimeDB();

        swipeEventosContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                getPlans(FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType());
                swipeEventosContainer.setRefreshing(false);
            }
        });

        swipeGruposContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                getPlans(FirebaseDBEnum.GRUPOS.getType(), FirebaseDBEnum.GRUPOS_LOCATION.getType());
                swipeGruposContainer.setRefreshing(false);
            }
        });

        //Tabs
        this.prepareTabs();

        return rootView;
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.clear();
        inflater.inflate(R.menu.main, menu);

        MenuItem favoriteItem = menu.findItem(R.id.action_search);
        Drawable newIcon = (Drawable)favoriteItem.getIcon();
        newIcon.mutate().setColorFilter(Color.WHITE, PorterDuff.Mode.SRC_IN);
        favoriteItem.setIcon(newIcon);
        newIcon.mutate().setColorFilter(getResources().getColor(R.color.white), PorterDuff.Mode.SRC_IN);

        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();

        if (id == R.id.action_search) {
            //Open dialog
            final Intent intent = new Intent(getActivity(), SearchActivity.class);
            intent.putExtra("distance", prefs.getInt("distance", 100));
            startActivityForResult(intent, SEARCH);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void prepareTabs(){
        tabs.setup();

        TabHost.TabSpec spec=tabs.newTabSpec(TiposPlanes.EVENTOS.getType());
        spec.setContent(R.id.tab_eventos);
        spec.setIndicator(getResources().getString(R.string.eventos));
        tabs.addTab(spec);

        spec=tabs.newTabSpec(TiposPlanes.GRUPOS.getType());
        spec.setContent(R.id.tab_grupos);
        spec.setIndicator(getResources().getString(R.string.grupos));
        tabs.addTab(spec);

        tabs.setOnTabChangedListener(this);

        final Bundle bundle = getArguments();
        if(bundle != null
            && bundle.containsKey("parentFragment")){
            final int type = bundle.getInt("parentFragment");
            if(type == AdapterEnum.HOME_EVENTOS.getId()){
                tabs.setCurrentTab(TiposPlanes.EVENTOS.getId());
            }else{
                tabs.setCurrentTab(TiposPlanes.GRUPOS.getId());
            }
        }else{
            tabs.setCurrentTab(TiposPlanes.EVENTOS.getId());
        }
    }

    private void getPlans(final String type, final String locationType){
        estaBuscando = true;
        if(FirebaseDBEnum.EVENTOS.getType().equals(type)){
            skeletonScreen = Skeleton.bind(eventosRV).adapter(eventosAdapter)
                    .load(R.layout.item_plan).show();
        }else{
            skeletonScreen = Skeleton.bind(gruposRV).adapter(gruposAdapter)
                    .load(R.layout.item_plan).show();
        }

        if (FirebaseDBEnum.GRUPOS.getType().equals(type)) {
            gruposAdapter.clearItems();
        } else {
            eventosAdapter.clearItems();
        }

        final Query queryPlans = realtimeDB.child(type).orderByChild("date/time").startAt(new Date().getTime());
        queryPlans.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                final GenericTypeIndicator<HashMap<String, Plan>> generic = new GenericTypeIndicator<HashMap<String, Plan>>() {};
                map = dataSnapshot.getValue(generic);

                if(map != null && !map.isEmpty()){
                    final GeoFire geoFire = new GeoFire(realtimeDB.child(locationType));
                    final String userId = prefs.getString("userId", "");
                    final Usuario usuario = daoSession.getUsuarioDao().load(userId);
                    int distance = prefs.getInt("distance", 100);
                    if(usuario.getLatitude() != null) {
                        final GeoQuery geoQuery = geoFire.queryAtLocation(new GeoLocation(usuario.getLatitude(),
                                usuario.getLongitude()), distance);
                        geoQuery.addGeoQueryEventListener(new GeoQueryEventListener() {
                            @Override
                            public void onKeyEntered(String key, GeoLocation location) {
                                if(map.containsKey(key)){
                                    if (FirebaseDBEnum.GRUPOS.getType().equals(type)) {
                                        gruposAdapter.addItem(map.get(key), type);
                                    } else {
                                        eventosAdapter.addItem(map.get(key), type);
                                    }
                                }
                            }

                            @Override
                            public void onKeyExited(String key) {
                                Log.d(TAG, "onKeyExited");
                            }

                            @Override
                            public void onKeyMoved(String key, GeoLocation location) {
                                Log.d(TAG, "onKeyMoved");
                            }

                            @Override
                            public void onGeoQueryReady() {
                                Log.d(TAG, "onGeoQueryReady");
                                skeletonScreen.hide();
                                estaBuscando = false;
                            }

                            @Override
                            public void onGeoQueryError(DatabaseError error) {
                                Log.d(TAG, "onGeoQueryError");
                                skeletonScreen.hide();
                                estaBuscando = false;
                            }

                        });
                    }else{
                        skeletonScreen.hide();
                        estaBuscando = false;
                    }
                }else{
                    skeletonScreen.hide();
                    estaBuscando = false;
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
                skeletonScreen.hide();
                estaBuscando = false;
            }
        });
    }

    @Override
    public void onTabChanged(String s) {
        if(s.equals(TiposPlanes.EVENTOS.getType())){
            this.getPlans(FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType());
        }else{
            this.getPlans(FirebaseDBEnum.GRUPOS.getType(), FirebaseDBEnum.GRUPOS_LOCATION.getType());
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case LOCATION_REQUEST_CODE: {
                if (ContextCompat.checkSelfPermission(getContext(), Manifest.permission.ACCESS_FINE_LOCATION)
                        == PackageManager.PERMISSION_GRANTED) {
                    //Get current location
                    FusedLocationProviderClient mFusedLocationClient = LocationServices.getFusedLocationProviderClient(getContext());
                    mFusedLocationClient.getLastLocation()
                            .addOnSuccessListener(new OnSuccessListener<Location>() {
                                @Override
                                public void onSuccess(Location location) {
                                    // Got last known location. In some rare situations this can be null.
                                    if (location != null) {
                                        enableLocation(location);
                                    }
                                }
                            });
                } else {
                    Log.d(TAG, "Deny permission");
                }
            }
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {

        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_CANCELED) {
            return;
        }
        if (requestCode == SEARCH) {
            if (data != null) {
                final int distance = data.getExtras().getInt("distance");
                SharedPreferences.Editor editor = prefs.edit();
                editor.putInt("distance", distance);
                editor.commit();

                if(tabs.getCurrentTab() == 0){
                    this.getPlans(FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType());
                }else{
                    this.getPlans(FirebaseDBEnum.GRUPOS.getType(), FirebaseDBEnum.GRUPOS_LOCATION.getType());
                }
            }
        }
    }


    @Override
    public void onDetach() {
        super.onDetach();
        if (mGoogleApiClient.isConnected()) {
            mGoogleApiClient.disconnect();
        }
    }

    @Override
    public void onConnected(@Nullable Bundle bundle) {
        if (ContextCompat.checkSelfPermission(getContext(), Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {
            requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, LOCATION_REQUEST_CODE);
        }else{
            //Get current location
            FusedLocationProviderClient mFusedLocationClient = LocationServices.getFusedLocationProviderClient(getContext());
            mFusedLocationClient.getLastLocation()
                    .addOnSuccessListener(new OnSuccessListener<Location>() {
                        @Override
                        public void onSuccess(Location location) {
                            // Got last known location. In some rare situations this can be null.
                            if (location != null) {
                                enableLocation(location);
                            }
                        }
                    });

        }
    }

    /**
     * Enable location.
     */
    private void enableLocation(Location location){
        //Get position if location exists
        if(location != null){
            final String userId = prefs.getString("userId", "");
            Map<String,Object> updates = new HashMap<>();

            double longitude = location.getLatitude();
            double latitude = location.getLongitude();
            //Para prevenir problemas con simuladores y localizaciones no existentes
            if(!GeneralUtils.coordinatesValid(longitude, latitude)){
                longitude = 43.2612776;
                latitude = -2.9007562;
            }

            updates.put("longitude", longitude);
            updates.put("latitude", latitude);

            realtimeDB.child(FirebaseDBEnum.USUARIOS.getType()).child(userId).updateChildren(updates);

            //Se actualiza el usuario local
            final Usuario usuarioSesion = daoSession.getUsuarioDao().load(userId);
            usuarioSesion.setLatitude(latitude);
            usuarioSesion.setLongitude(longitude);
            daoSession.getUsuarioDao().update(usuarioSesion);

            if(!estaBuscando){
                if(tabs.getCurrentTab() == 0){
                    this.getPlans(FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType());
                }else{
                    this.getPlans(FirebaseDBEnum.GRUPOS.getType(), FirebaseDBEnum.GRUPOS_LOCATION.getType());
                }
            }
        }
    }

    @Override
    public void onConnectionSuspended(int i) {

    }

    @Override
    public void onConnectionFailed(@NonNull ConnectionResult connectionResult) {

    }

    @Override
    public void onLocationChanged(Location location) {
        enableLocation(location);
    }

    private void switchContent(Fragment fragment) {
        MainActivity mainActivity = (MainActivity) getActivity();
        mainActivity.switchFragmentToUp(fragment);
    }

}
