package edu.uoc.android.socialplan.fragments;

import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ethanhua.skeleton.Skeleton;
import com.ethanhua.skeleton.SkeletonScreen;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.adapters.NotificacionesRecyclerViewAdapter;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.models.Notificacion;

/**
 * Created by alejandroresuagarcia on 13/11/17.
 */

public class NotificacionesFragment extends Fragment {

    @BindView(R.id.notificaciones_recycler_view) RecyclerView notificacionesRV;
    @BindView(R.id.swipe_notificaciones_container) SwipeRefreshLayout swipeNotificacionesContainer;

    private static final String TAG = "NotificacionesFragment";

    private SharedPreferences prefs;
    private DatabaseReference realtimeDB;

    private SkeletonScreen skeletonScreen;

    private NotificacionesRecyclerViewAdapter notificacionesAdapter;
    private List<Notificacion> lista;

    public NotificacionesFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(false);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        final View rootView = inflater.inflate(R.layout.fragment_notificaciones, container, false);
        ButterKnife.bind(this, rootView);

        ((MainActivity)getActivity()).getSupportActionBar().setTitle(getResources().getString(R.string.notificaciones));
        prefs = ((App) getActivity().getApplication()).getPrefs();
        realtimeDB = ((App) getActivity().getApplication()).getRealtimeDB();

        this.notificacionesAdapter = new NotificacionesRecyclerViewAdapter(getContext(), new ArrayList<Notificacion>());
        notificacionesRV.setAdapter(this.notificacionesAdapter);

        getNotificaciones();

        swipeNotificacionesContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                getNotificaciones();
                swipeNotificacionesContainer.setRefreshing(false);
            }
        });

        return rootView;
    }

    private void getNotificaciones(){
        skeletonScreen = Skeleton.bind(notificacionesRV).adapter(notificacionesAdapter)
                    .load(R.layout.item_plan).show();
        notificacionesAdapter.clearItems();

        final String userId = prefs.getString("userId", "");
        final Query queryPlans = realtimeDB
                .child(FirebaseDBEnum.NOTIFICACIONES.getType())
                .orderByChild("name")
                .equalTo(userId);

        queryPlans.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                lista = new ArrayList<>();
                for (DataSnapshot postSnapshot : dataSnapshot.getChildren()) {
                    Notificacion notificacion = postSnapshot.getValue(Notificacion.class);
                    lista.add(notificacion);
                }
                if (!lista.isEmpty()) {
                    Collections.sort(lista, new Comparator<Notificacion>() {
                        public int compare(Notificacion o1, Notificacion o2) {
                            return o1.getDate().compareTo(o2.getDate());
                        }
                    });
                    notificacionesAdapter.addItem(lista);
                }
                skeletonScreen.hide();
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
                skeletonScreen.hide();
            }
        });
    }

}
