package edu.uoc.android.socialplan.fragments;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.location.Address;
import android.location.Geocoder;
import android.net.Uri;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v4.content.res.ResourcesCompat;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Switch;
import android.widget.TextView;

import com.firebase.geofire.GeoFire;
import com.firebase.geofire.GeoLocation;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DatabaseReference;
import com.kunzisoft.switchdatetime.SwitchDateTimeDialogFragment;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.activities.planes.PlacesSearchActivity;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.constants.TiposPlanes;
import edu.uoc.android.socialplan.models.DaoSession;
import edu.uoc.android.socialplan.models.Plan;
import edu.uoc.android.socialplan.models.Usuario;
import edu.uoc.android.socialplan.utils.ImageUtils;
import edu.uoc.android.socialplan.utils.ValidacionUtils;
import mabbas007.tagsedittext.TagsEditText;

/**
 * Created by alejandroresuagarcia on 13/11/17.
 */

public class NuevoPlanFragment extends Fragment {

    @BindView(R.id.plan_image) ImageView planImage;
    @BindView(R.id.plan_title) EditText planTitle;
    @BindView(R.id.plan_description) EditText planDescription;
    @BindView(R.id.plan_place) TextView planPlace;
    @BindView(R.id.plan_date) TextView planDate;
    @BindView(R.id.plan_switch) Switch planSwitch;
    @BindView(R.id.plan_users_count) EditText planUsersCount;
    @BindView(R.id.hashtag_edittext) TagsEditText planTags;

    private static final String TAG = "NuevoPlanFragment";

    private int GALLERY = 1, CAMERA = 2, SEARCH = 3;
    private static final int MY_CAMERA_REQUEST_CODE = 100;

    private String userImageB64;
    private FirebaseUser user;
    private GeoFire geoFire;
    private DatabaseReference realtimeDB;
    private DaoSession greendaoDb;

    private double latitud;
    private double longitud;
    private boolean esGrupo = false;
    private Date fecha;
    private Boolean esNavigation = false;

    public NuevoPlanFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        final View rootView = inflater.inflate(R.layout.fragment_nuevo_plan, container, false);
        ButterKnife.bind(this, rootView);

        if(getArguments() != null
                && getArguments().containsKey("navigation")
                && getArguments().getBoolean("navigation")){
            Log.d(TAG, "Navigation control");
            esNavigation = true;

            getActivity().invalidateOptionsMenu();

            AppCompatActivity activity = (AppCompatActivity) getActivity();
            activity.setSupportActionBar(((MainActivity)getActivity()).toolbar);

            ((MainActivity)getActivity()).toggle = new ActionBarDrawerToggle(
                    ((MainActivity)getActivity()), ((MainActivity)getActivity()).drawer, ((MainActivity)getActivity()).toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
            ((MainActivity)getActivity()).drawer.addDrawerListener(((MainActivity)getActivity()).toggle);
            ((MainActivity)getActivity()).toggle.syncState();
        }else{
            Drawable drawable = ResourcesCompat.getDrawable(getResources(), android.R.drawable.ic_menu_close_clear_cancel, null);
            drawable = DrawableCompat.wrap(drawable);
            DrawableCompat.setTint(drawable, Color.WHITE);

            ((MainActivity)getActivity()).toggle.setDrawerIndicatorEnabled(false);
            ((MainActivity)getActivity()).toggle.setHomeAsUpIndicator(drawable);
            ((MainActivity)getActivity()).toggle.syncState();

            AppCompatActivity activity = (AppCompatActivity) getActivity();
            activity.setSupportActionBar(((MainActivity)getActivity()).toolbar);
            activity.getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        }
        ((MainActivity)getActivity()).getSupportActionBar().setTitle(getResources().getString(R.string.nuevo_plan));

        user = FirebaseAuth.getInstance().getCurrentUser();
        geoFire = ((App)getActivity().getApplication()).getGeoFire();
        realtimeDB = ((App)getActivity().getApplication()).getRealtimeDB();
        greendaoDb = ((App)getActivity().getApplication()).getDaoSession();

        final Usuario usuario = greendaoDb.getUsuarioDao().load(user.getUid());
        if(usuario.getLatitude() != null
                && usuario.getLatitude() != 0){
            final Geocoder geo = new Geocoder(getContext());
            try {
                final List<Address> lista = geo.getFromLocation(usuario.getLongitude(), usuario.getLatitude(), 1);
                if(lista != null
                        && !lista.isEmpty()){
                    planPlace.setText(lista.get(0).getLocality());
                    longitud = usuario.getLatitude();
                    latitud = usuario.getLongitude();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        planSwitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                esGrupo = b;
                if(esGrupo){
                    planUsersCount.setVisibility(View.VISIBLE);
                }else{
                    planUsersCount.setVisibility(View.GONE);
                }
            }
        });

        //Se introduce la fecha de hoy
        final Calendar c = Calendar.getInstance();
        c.add(Calendar.DATE, 1);
        fecha = c.getTime();
        planDate.setText(dateToString(c.getTime()));

        planDescription.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                planDescription.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        planTitle.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                planTitle.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });

        return rootView;
    }

    private String dateToString(Date date) {
        final String myFormat = getResources().getString(R.string.date_mask); //In which you need put here
        SimpleDateFormat sdf = new SimpleDateFormat(myFormat, new Locale("es"));
        return sdf.format(date);
    }

    private void updateLabel(Date date) {
        planDate.setText(dateToString(date));
        fecha = date;
    }

    @OnClick(R.id.plan_image)
    public void userImageButton(View view) {
        AlertDialog.Builder pictureDialog = new AlertDialog.Builder(getContext());
        pictureDialog.setTitle(getResources().getString(R.string.seleccionar_opcion));
        String[] pictureDialogItems = {
                getResources().getString(R.string.seleccionar_foto),
                getResources().getString(R.string.camara) };
        pictureDialog.setItems(pictureDialogItems, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case 0:
                        choosePhotoFromGallery();
                        break;
                    case 1:
                        takePhotoFromCamera();
                        break;
                }
            }
        });
        pictureDialog.show();
    }

    public void choosePhotoFromGallery() {
        final Intent photoPickerIntent = new Intent(Intent.ACTION_GET_CONTENT);
        photoPickerIntent.setType("image/*");
        startActivityForResult(photoPickerIntent, GALLERY);
    }

    private void takePhotoFromCamera() {
        if (ContextCompat.checkSelfPermission(getContext(), Manifest.permission.CAMERA)
                == PackageManager.PERMISSION_DENIED){
            requestPermissions(new String[] {Manifest.permission.CAMERA}, MY_CAMERA_REQUEST_CODE);
        }else{
            final Intent intent = new Intent(android.provider.MediaStore.ACTION_IMAGE_CAPTURE);
            startActivityForResult(intent, CAMERA);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case MY_CAMERA_REQUEST_CODE: {
                if (ContextCompat.checkSelfPermission(getContext(), Manifest.permission.CAMERA)
                        == PackageManager.PERMISSION_GRANTED) {
                    final Intent intent = new Intent(android.provider.MediaStore.ACTION_IMAGE_CAPTURE);
                    startActivityForResult(intent, CAMERA);
                } else {
                    Log.d(TAG, "Deny permission");
                }
            }
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {

        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_CANCELED) {
            return;
        }
        if (requestCode == GALLERY) {
            if (data != null) {
                Uri contentURI = data.getData();
                try {
                    final InputStream imageStream = getContext().getContentResolver().openInputStream(contentURI);
                    Bitmap bitmap = BitmapFactory.decodeStream(imageStream);
                    bitmap=Bitmap.createScaledBitmap(bitmap, 200, 200, true);
                    planImage.setImageBitmap(bitmap);
                    userImageB64 = ImageUtils.convert(bitmap);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        } else if (requestCode == CAMERA) {
            Bitmap thumbnail = (Bitmap) data.getExtras().get("data");
            planImage.setImageBitmap(thumbnail);
            userImageB64 = ImageUtils.convert(thumbnail);
        }else if(requestCode == SEARCH){
            String lugar = (String)data.getExtras().get("lugar");
            planPlace.setText(lugar);
            latitud = (double)data.getExtras().get("latitud");
            longitud = (double)data.getExtras().get("longitud");

            Log.d(TAG, lugar);
        }
    }

    public void crearNuevoEvento() {
        boolean error = false;
        if(!ValidacionUtils.isRequired(planTitle.getText().toString())){
            ValidacionUtils.hayErrorEditText(getResources(), planTitle);
            error = true;
        }
        if(!ValidacionUtils.isRequired(planDescription.getText().toString())){
            ValidacionUtils.hayErrorEditText(getResources(), planDescription);
            error = true;
        }
        if(!ValidacionUtils.isRequired(planPlace.getText().toString())){
            ValidacionUtils.hayErrorTextView(getResources(), planPlace);
            error = true;
        }
        if(!ValidacionUtils.isRequired(planDate.getText().toString())){
            ValidacionUtils.hayErrorTextView(getResources(), planDate);
            error = true;
        }

        if(!error){
            final Plan plan = new Plan();
            plan.setImage(userImageB64);
            plan.setDescription(planDescription.getText().toString());
            plan.setTitle(planTitle.getText().toString());
            plan.setUserId(user.getUid());
            plan.setDate(fecha);
            plan.setUsersCount(1);
            plan.setHash(planTags.getTags());

            String tipoPlan = FirebaseDBEnum.EVENTOS.getType();
            String tipoPlanLocation = FirebaseDBEnum.EVENTOS_LOCATION.getType();
            if(esGrupo){
                plan.setPlanType(TiposPlanes.GRUPOS.getId());
                if(planUsersCount != null
                        && planUsersCount.getText() != null
                        && !"".equals(planUsersCount.getText().toString())){
                    plan.setUsersMax(Integer.valueOf(planUsersCount.getText().toString()));
                }
                tipoPlan = FirebaseDBEnum.GRUPOS.getType();
                tipoPlanLocation = FirebaseDBEnum.GRUPOS_LOCATION.getType();
            }else{
                plan.setPlanType(TiposPlanes.EVENTOS.getId());
            }

            String planId = realtimeDB.child(tipoPlan).push().getKey();
            plan.setId(planId);
            realtimeDB.child(tipoPlan).child(planId).setValue(plan);
            geoFire = new GeoFire(realtimeDB.child(tipoPlanLocation));
            geoFire.setLocation(planId, new GeoLocation(longitud, latitud));

            greendaoDb.getPlanDao().insertOrReplace(plan);

            switchContent(new HomeFragment());
        }else{
            Snackbar.make(getView(), getResources().getString(R.string.validacion_error), Snackbar.LENGTH_LONG).show();
        }
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.clear();
        getActivity().getMenuInflater().inflate(R.menu.menu_dialog, menu);

        MenuItem favoriteItem = menu.findItem(R.id.action_save);
        Drawable newIcon = (Drawable)favoriteItem.getIcon();
        newIcon.mutate().setColorFilter(Color.WHITE, PorterDuff.Mode.SRC_IN);
        favoriteItem.setIcon(newIcon);
        newIcon.mutate().setColorFilter(getResources().getColor(R.color.white), PorterDuff.Mode.SRC_IN);

        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();

        if (id == R.id.action_save) {
            // handle confirmation button click here
            crearNuevoEvento();
            return true;
        } else if(!esNavigation
                && id == android.R.id.home) {
            // handle close button click here
            switchContent(new HomeFragment());
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void switchContent(Fragment fragment) {
        MainActivity mainActivity = (MainActivity) getContext();
        mainActivity.switchFragment(fragment);
    }

    @OnClick(R.id.plan_place_layout)
    public void planPlaceLayout(View view) {
        final Intent intent = new Intent(getActivity(), PlacesSearchActivity.class);
        startActivityForResult(intent, SEARCH);
    }

    @OnClick(R.id.plan_date_layout)
    public void planDateLayout(View view) {
        // Initialize
        final SwitchDateTimeDialogFragment dateTimeDialogFragment = SwitchDateTimeDialogFragment.newInstance(
                "",
                getResources().getString(R.string.aceptar),
                getResources().getString(R.string.cancelar)
        );

        // Assign values
        dateTimeDialogFragment.startAtCalendarView();
        dateTimeDialogFragment.set24HoursMode(true);
        dateTimeDialogFragment.setDefaultDateTime(new Date());
        dateTimeDialogFragment.setDefaultHourOfDay(0);
        dateTimeDialogFragment.setDefaultMinute(0);

        // Set listener
        dateTimeDialogFragment.setOnButtonClickListener(new SwitchDateTimeDialogFragment.OnButtonClickListener() {
            @Override
            public void onPositiveButtonClick(Date date) {
                updateLabel(date);
            }

            @Override
            public void onNegativeButtonClick(Date date) {
                dateTimeDialogFragment.dismiss();
            }
        });

        dateTimeDialogFragment.show(getFragmentManager(), "dialog_time");
    }

}
