package edu.uoc.android.socialplan.fragments;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.constants.TipoLoginUserEnum;
import edu.uoc.android.socialplan.constants.TiposPlanes;
import edu.uoc.android.socialplan.models.DaoSession;
import edu.uoc.android.socialplan.models.Plan;
import edu.uoc.android.socialplan.models.PlanDao;
import edu.uoc.android.socialplan.models.Usuario;
import edu.uoc.android.socialplan.utils.GeneralUtils;
import edu.uoc.android.socialplan.utils.ImageUtils;
import edu.uoc.android.socialplan.utils.ValidacionUtils;
import edu.uoc.android.socialplan.views.CustomProgressBarDialog;

/**
 * Created by alejandroresuagarcia on 13/11/17.
 */

public class PerfilFragment extends Fragment {

    @BindView(R.id.user_image) ImageView userImage;
    @BindView(R.id.user_name) EditText userName;
    @BindView(R.id.user_mail) TextView userMail;
    @BindView(R.id.eventos_count) TextView eventosCountTV;
    @BindView(R.id.grupos_count) TextView gruposCountTV;
    @BindView(R.id.favoritos_count) TextView favoritosCountTV;

    private DaoSession daoSession;
    private DatabaseReference realtimeDB;

    private static String TAG = "PerfilFragment";
    private int GALLERY = 1, CAMERA = 2;
    private static final int MY_CAMERA_REQUEST_CODE = 100;
    private String userImageB64;
    private Usuario usuario;
    private CustomProgressBarDialog progressBarDialog;

    private Map<String, Plan> map = new HashMap<String, Plan>();

    public PerfilFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        final View rootView = inflater.inflate(R.layout.fragment_perfil, container, false);
        ButterKnife.bind(this, rootView);

        ((MainActivity)getActivity()).getSupportActionBar().setTitle(getResources().getString(R.string.perfil));

        daoSession = ((App)getActivity().getApplication()).getDaoSession();
        final SharedPreferences prefs = ((App) getActivity().getApplication()).getPrefs();
        realtimeDB = ((App) getActivity().getApplication()).getRealtimeDB();
        final String userId = prefs.getString("userId", "");
        usuario = daoSession.getUsuarioDao().load(userId);
        this.progressBarDialog = GeneralUtils.getCustomProgressBarDialog(getContext());
        progressBarDialog.show();

        if (usuario.getPhoto() != null) {
            if(usuario.getLoginType() != null
                    && usuario.getLoginType() == TipoLoginUserEnum.FACEBOOK.getId()) {
                Picasso.with(getContext()).load(usuario.getPhoto()).into(userImage);
            }else{
                final Bitmap bitmap = ImageUtils.convert(usuario.getPhoto());
                userImage.setImageBitmap(bitmap);
            }
        }
        userName.setText(usuario.getName());
        userMail.setText(usuario.getMail());

        userName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                userName.setCompoundDrawables(null,null,null,null);
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });

        getCounts();

        return rootView;
    }

    private void getCounts(){
        long eventosCount = daoSession.getPlanDao().queryBuilder().where(PlanDao.Properties.PlanType.eq(TiposPlanes.EVENTOS.getId())).count();
        long gruposCount = daoSession.getPlanDao().queryBuilder().where(PlanDao.Properties.PlanType.eq(TiposPlanes.GRUPOS.getId())).count();

        eventosCountTV.setText(String.valueOf(eventosCount));
        gruposCountTV.setText(String.valueOf(gruposCount));

        Query queryFavoritos = realtimeDB.child(FirebaseDBEnum.EVENTOS.getType()).orderByChild("usuarios/"+usuario.getId()).equalTo(true);
        //Query queryAsistencia = realtimeDB.child(FirebaseDBEnum.GRUPOS.getType()).orderByChild("usuarios/"+usuario.getId()).equalTo(true);

        queryFavoritos.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                favoritosCountTV.setText(String.valueOf(dataSnapshot.getChildrenCount()));
                progressBarDialog.hide();
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
                progressBarDialog.hide();
            }
        });
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.clear();
        inflater.inflate(R.menu.menu_perfil, menu);

        MenuItem favoriteItem = menu.findItem(R.id.action_save);
        Drawable newIcon = (Drawable)favoriteItem.getIcon();
        newIcon.mutate().setColorFilter(Color.WHITE, PorterDuff.Mode.SRC_IN);
        favoriteItem.setIcon(newIcon);
        newIcon.mutate().setColorFilter(getResources().getColor(R.color.white), PorterDuff.Mode.SRC_IN);

        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();

        if (id == R.id.action_save) {
            modifyOpenDialog();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void modifyOpenDialog(){
        AlertDialog.Builder builder;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            builder = new AlertDialog.Builder(getContext(), android.R.style.Theme_Material_Dialog_Alert);
        } else {
            builder = new AlertDialog.Builder(getContext());
        }
        builder.setTitle(getResources().getString(R.string.informacion))
                .setMessage(getResources().getString(R.string.modificar_info))
                .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        modify();
                    }
                })
                .setNegativeButton(getResources().getString(R.string.cancelar), new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        //Do not anything
                    }
                })
                .setIcon(android.R.drawable.ic_dialog_alert)
                .show();
    }

    private void modify(){
        progressBarDialog.show();
        boolean error = false;
        if(!ValidacionUtils.isRequired(userName.getText().toString())){
            ValidacionUtils.hayErrorEditText(getResources(), userName);
            Snackbar.make(getView(), getResources().getString(R.string.validacion_error), Snackbar.LENGTH_LONG).show();
            error = true;
        }
        if(!error){
            usuario.setName(userName.getText().toString());
            realtimeDB.child(FirebaseDBEnum.USUARIOS.getType()).child(usuario.getId()).updateChildren(usuario.toMap());

            daoSession.getUsuarioDao().update(usuario);

            Snackbar.make(getView(), getResources().getString(R.string.datos_actualizados), Snackbar.LENGTH_LONG).show();
            ((MainActivity)getActivity()).userName.setText(usuario.getName());
        }
        progressBarDialog.hide();
    }

    @OnClick(R.id.user_image)
    public void userImageButton(View view) {
        AlertDialog.Builder pictureDialog = new AlertDialog.Builder(getContext());
        pictureDialog.setTitle(getResources().getString(R.string.seleccionar_opcion));
        String[] pictureDialogItems = {
                getResources().getString(R.string.seleccionar_foto),
                getResources().getString(R.string.camara) };
        pictureDialog.setItems(pictureDialogItems, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case 0:
                        choosePhotoFromGallery();
                        break;
                    case 1:
                        takePhotoFromCamera();
                        break;
                }
            }
        });
        pictureDialog.show();
    }

    public void choosePhotoFromGallery() {
        final Intent photoPickerIntent = new Intent(Intent.ACTION_GET_CONTENT);
        photoPickerIntent.setType("image/*");
        startActivityForResult(photoPickerIntent, GALLERY);
    }

    private void takePhotoFromCamera() {
        if (ContextCompat.checkSelfPermission(getContext(), Manifest.permission.CAMERA)
                == PackageManager.PERMISSION_DENIED){
            requestPermissions(new String[] {Manifest.permission.CAMERA}, MY_CAMERA_REQUEST_CODE);
        }else{
            final Intent intent = new Intent(android.provider.MediaStore.ACTION_IMAGE_CAPTURE);
            startActivityForResult(intent, CAMERA);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case MY_CAMERA_REQUEST_CODE: {
                if (ContextCompat.checkSelfPermission(getContext(), Manifest.permission.CAMERA)
                        == PackageManager.PERMISSION_GRANTED) {
                    final Intent intent = new Intent(android.provider.MediaStore.ACTION_IMAGE_CAPTURE);
                    startActivityForResult(intent, CAMERA);
                } else {
                    Log.d(TAG, "Deny permission");
                }
            }
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {

        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_CANCELED) {
            return;
        }
        if (requestCode == GALLERY) {
            if (data != null) {
                Uri contentURI = data.getData();
                try {
                    final InputStream imageStream = getContext().getContentResolver().openInputStream(contentURI);
                    Bitmap bitmap = BitmapFactory.decodeStream(imageStream);
                    bitmap=Bitmap.createScaledBitmap(bitmap, 200, 200, true);
                    userImage.setImageBitmap(bitmap);
                    userImageB64 = ImageUtils.convert(bitmap);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        } else if (requestCode == CAMERA) {
            Bitmap thumbnail = (Bitmap) data.getExtras().get("data");
            userImage.setImageBitmap(thumbnail);
            userImageB64 = ImageUtils.convert(thumbnail);
        }
        if (userImageB64 != null) {
            usuario.setPhoto(userImageB64);
            if (usuario.getLoginType() != null &&
                    usuario.getLoginType() == TipoLoginUserEnum.FACEBOOK.getId()) {
                usuario.setLoginType(TipoLoginUserEnum.MODIFY.getId());
            }
        }
        realtimeDB.child(FirebaseDBEnum.USUARIOS.getType()).child(usuario.getId()).updateChildren(usuario.toMap());
        daoSession.getUsuarioDao().update(usuario);
        Snackbar.make(getView(), getResources().getString(R.string.datos_actualizados), Snackbar.LENGTH_LONG).show();

        if(usuario.getPhoto() != null){
            if(usuario.getLoginType() != null
                    && usuario.getLoginType() == TipoLoginUserEnum.FACEBOOK.getId()) {
                Picasso.with(getContext()).load(usuario.getPhoto()).into(((MainActivity)getActivity()).userImage);
            }else{
                final Bitmap bitmap = ImageUtils.convert(usuario.getPhoto());
                ((MainActivity)getActivity()).userImage.setImageBitmap(bitmap);
            }
        }
    }


}
