package edu.uoc.android.socialplan.fragments;

import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.helper.ItemTouchHelper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TabHost;

import com.ethanhua.skeleton.Skeleton;
import com.ethanhua.skeleton.SkeletonScreen;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import edu.uoc.android.socialplan.R;
import edu.uoc.android.socialplan.activities.general.MainActivity;
import edu.uoc.android.socialplan.adapters.PlanesRecyclerViewAdapter;
import edu.uoc.android.socialplan.adapters.RecyclerItemTouchHelper;
import edu.uoc.android.socialplan.application.App;
import edu.uoc.android.socialplan.constants.FirebaseDBEnum;
import edu.uoc.android.socialplan.constants.AdapterEnum;
import edu.uoc.android.socialplan.constants.TiposPlanes;
import edu.uoc.android.socialplan.holders.PlanHolder;
import edu.uoc.android.socialplan.models.Plan;

/**
 * Created by alejandroresuagarcia on 13/11/17.
 */

public class RedFragment extends Fragment implements TabHost.OnTabChangeListener, RecyclerItemTouchHelper.RecyclerItemTouchHelperListener {

    @BindView(R.id.fab) FloatingActionButton fab;
    @BindView(R.id.tab_host) TabHost tabs;
    @BindView(R.id.grupos_recycler_view) RecyclerView gruposRV;
    @BindView(R.id.eventos_recycler_view) RecyclerView eventosRV;
    @BindView(R.id.favs_recycler_view) RecyclerView favsRV;
    @BindView(R.id.swipe_grupos_container) SwipeRefreshLayout swipeGruposContainer;
    @BindView(R.id.swipe_eventos_container) SwipeRefreshLayout swipeEventosContainer;
    @BindView(R.id.swipe_favs_container) SwipeRefreshLayout swipeFavsContainer;

    private static final String TAG = "HomeFragment";

    private SharedPreferences prefs;
    private DatabaseReference realtimeDB;

    private SkeletonScreen skeletonScreen;

    private PlanesRecyclerViewAdapter gruposAdapter;
    private PlanesRecyclerViewAdapter eventosAdapter;
    private PlanesRecyclerViewAdapter favsAdapter;
    private Map<String, Plan> map = new HashMap<String, Plan>();

    public RedFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(false);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        final View rootView = inflater.inflate(R.layout.fragment_red, container, false);
        ButterKnife.bind(this, rootView);

        AppCompatActivity activity = (AppCompatActivity) getActivity();
        activity.setSupportActionBar(((MainActivity)getActivity()).toolbar);

        ((MainActivity)getActivity()).toggle = new ActionBarDrawerToggle(
                ((MainActivity)getActivity()), ((MainActivity)getActivity()).drawer, ((MainActivity)getActivity()).toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        ((MainActivity)getActivity()).drawer.addDrawerListener(((MainActivity)getActivity()).toggle);
        ((MainActivity)getActivity()).toggle.syncState();
        ((MainActivity)getActivity()).getSupportActionBar().setTitle(getResources().getString(R.string.red));

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                switchContent(new NuevoPlanFragment());
            }
        });

        prefs = ((App) getActivity().getApplication()).getPrefs();
        realtimeDB = ((App) getActivity().getApplication()).getRealtimeDB();

        this.gruposAdapter = new PlanesRecyclerViewAdapter(getContext(), new ArrayList<Plan>(),
                FirebaseDBEnum.GRUPOS.getType(), FirebaseDBEnum.GRUPOS_LOCATION.getType(), AdapterEnum.RED_GRUPOS.getId());
        gruposRV.setAdapter(this.gruposAdapter);

        this.eventosAdapter = new PlanesRecyclerViewAdapter(getContext(), new ArrayList<Plan>(),
                FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType(), AdapterEnum.RED_EVENTOS.getId());
        eventosRV.setAdapter(this.eventosAdapter);

        this.favsAdapter = new PlanesRecyclerViewAdapter(getContext(), new ArrayList<Plan>(),
                FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType(), AdapterEnum.RED_FAVS.getId());
        favsRV.setAdapter(this.favsAdapter);

        swipeGruposContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                getPlans(FirebaseDBEnum.GRUPOS.getType(), FirebaseDBEnum.GRUPOS_LOCATION.getType(), FirebaseDBEnum.GRUPOS.getType());
                swipeGruposContainer.setRefreshing(false);
            }
        });

        swipeEventosContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                getPlans(FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType(), FirebaseDBEnum.EVENTOS.getType());
                swipeEventosContainer.setRefreshing(false);
            }
        });

        swipeFavsContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                getPlans(FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType(), "favoritos");
                swipeFavsContainer.setRefreshing(false);
            }
        });

        RecyclerView.LayoutManager gruposLayoutManager = new LinearLayoutManager(getContext());
        gruposRV.setLayoutManager(gruposLayoutManager);
        gruposRV.setItemAnimator(new DefaultItemAnimator());
        gruposRV.addItemDecoration(new DividerItemDecoration(getContext(), DividerItemDecoration.VERTICAL));
        gruposRV.setAdapter(gruposAdapter);
        ItemTouchHelper.SimpleCallback gruposCall = new RecyclerItemTouchHelper(0, ItemTouchHelper.LEFT, this);
        new ItemTouchHelper(gruposCall).attachToRecyclerView(gruposRV);

        RecyclerView.LayoutManager eventosManager = new LinearLayoutManager(getContext());
        eventosRV.setLayoutManager(eventosManager);
        eventosRV.setItemAnimator(new DefaultItemAnimator());
        eventosRV.addItemDecoration(new DividerItemDecoration(getContext(), DividerItemDecoration.VERTICAL));
        eventosRV.setAdapter(eventosAdapter);
        ItemTouchHelper.SimpleCallback eventosCall = new RecyclerItemTouchHelper(0, ItemTouchHelper.LEFT, this);
        new ItemTouchHelper(eventosCall).attachToRecyclerView(eventosRV);

        //Tabs
        this.prepareTabs();

        return rootView;
    }

    /**
     * callback when recycler view is swiped
     * item will be removed on swiped
     * undo option will be provided in snackbar to restore the item
     */
    @Override
    public void onSwiped(RecyclerView.ViewHolder viewHolder, int direction, int position) {
        if (viewHolder instanceof PlanHolder) {
            if(tabs.getCurrentTab() == 0){
                gruposAdapter.removeItem(viewHolder.getAdapterPosition());
            }else{
                eventosAdapter.removeItem(viewHolder.getAdapterPosition());
            }
        }
    }

    private void getPlans(final String type, final String locationType, final String s){
        if(FirebaseDBEnum.EVENTOS.getType().equals(s)){
            skeletonScreen = Skeleton.bind(eventosRV).adapter(eventosAdapter)
                    .load(R.layout.item_plan).show();
        }else if(FirebaseDBEnum.GRUPOS.getType().equals(s)){
            skeletonScreen = Skeleton.bind(gruposRV).adapter(gruposAdapter)
                    .load(R.layout.item_plan).show();
        }else if("favoritos".equals(s)){
            skeletonScreen = Skeleton.bind(favsRV).adapter(favsAdapter)
                    .load(R.layout.item_plan).show();
        }

        if (FirebaseDBEnum.GRUPOS.getType().equals(s)) {
            gruposAdapter.clearItems();
        } else if (FirebaseDBEnum.EVENTOS.getType().equals(s)) {
            eventosAdapter.clearItems();
        }else if("favoritos".equals(s)){
            favsAdapter.clearItems();
        }

        final String userId = prefs.getString("userId", "");

        Query queryPlans;

        if("favoritos".equals(s)){
            queryPlans = realtimeDB
                    .child(type)
                    .orderByChild("usuarios/"+userId)
                    .equalTo(true);
        }else{
            queryPlans = realtimeDB
                    .child(type)
                    .orderByChild("userId")
                    .equalTo(userId);
        }

        queryPlans.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                final GenericTypeIndicator<HashMap<String, Plan>> generic = new GenericTypeIndicator<HashMap<String, Plan>>() {};
                map = dataSnapshot.getValue(generic);

                if(map != null && !map.isEmpty()){
                    for(Map.Entry item: map.entrySet()){
                        if (FirebaseDBEnum.GRUPOS.getType().equals(s)) {
                            gruposAdapter.addItem((Plan) item.getValue(), type);
                        } else if (FirebaseDBEnum.EVENTOS.getType().equals(s)){
                            eventosAdapter.addItem((Plan) item.getValue(), type);
                        }else if ("favoritos".equals(s)){
                            favsAdapter.addItem((Plan) item.getValue(), type);
                        }
                    }
                }
                skeletonScreen.hide();
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("onCancelled");
                skeletonScreen.hide();
            }
        });
    }

    private void prepareTabs(){
        tabs.setup();

        TabHost.TabSpec spec=tabs.newTabSpec(TiposPlanes.GRUPOS.getType());
        spec.setContent(R.id.tab_grupos);
        spec.setIndicator(getResources().getString(R.string.grupos));
        tabs.addTab(spec);

        spec=tabs.newTabSpec(TiposPlanes.EVENTOS.getType());
        spec.setContent(R.id.tab_eventos);
        spec.setIndicator(getResources().getString(R.string.eventos));
        tabs.addTab(spec);

        spec=tabs.newTabSpec("favoritos");
        spec.setContent(R.id.tab_favoritos);
        spec.setIndicator(getResources().getString(R.string.favoritos));
        tabs.addTab(spec);

        tabs.setOnTabChangedListener(this);

        final Bundle bundle = getArguments();
        if(bundle != null
                && bundle.containsKey("parentFragment")){
            final int type = bundle.getInt("parentFragment");
            if(type == AdapterEnum.RED_GRUPOS.getId()){
                tabs.setCurrentTab(1);
                tabs.setCurrentTab(0);
            }else if(type == AdapterEnum.RED_EVENTOS.getId()){
                tabs.setCurrentTab(1);
            }else if(type == AdapterEnum.RED_FAVS.getId()){
                tabs.setCurrentTab(2);
            }
        }else{
            tabs.setCurrentTab(1);
            tabs.setCurrentTab(0);
        }
    }

    @Override
    public void onTabChanged(String s) {
        if(s.equals(TiposPlanes.GRUPOS.getType())){
            this.getPlans(FirebaseDBEnum.GRUPOS.getType(), FirebaseDBEnum.GRUPOS_LOCATION.getType(), s);
        }else if(s.equals(TiposPlanes.EVENTOS.getType())){
            this.getPlans(FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType(), s);
        }else if(s.equals("favoritos")){
            this.getPlans(FirebaseDBEnum.EVENTOS.getType(), FirebaseDBEnum.EVENTOS_LOCATION.getType(), s);
        }
    }

    private void switchContent(Fragment fragment) {
        MainActivity mainActivity = (MainActivity) getActivity();
        mainActivity.switchFragmentToUp(fragment);
    }

}
